% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/portion.R, R/rsi_df.R
\name{portion}
\alias{portion}
\alias{portion_R}
\alias{portion_IR}
\alias{portion_I}
\alias{portion_SI}
\alias{portion_S}
\alias{portion_df}
\alias{rsi_df}
\title{Calculate resistance of isolates}
\source{
\strong{M39 Analysis and Presentation of Cumulative Antimicrobial Susceptibility Test Data, 4th Edition}, 2014, \emph{Clinical and Laboratory Standards Institute (CLSI)}. \url{https://clsi.org/standards/products/microbiology/documents/m39/}.

Wickham H. \strong{Tidy Data.} The Journal of Statistical Software, vol. 59, 2014. \url{http://vita.had.co.nz/papers/tidy-data.html}
}
\usage{
portion_R(..., minimum = 30, as_percent = FALSE,
  only_all_tested = FALSE)

portion_IR(..., minimum = 30, as_percent = FALSE,
  only_all_tested = FALSE)

portion_I(..., minimum = 30, as_percent = FALSE,
  only_all_tested = FALSE)

portion_SI(..., minimum = 30, as_percent = FALSE,
  only_all_tested = FALSE)

portion_S(..., minimum = 30, as_percent = FALSE,
  only_all_tested = FALSE)

portion_df(data, translate_ab = "name", language = get_locale(),
  minimum = 30, as_percent = FALSE, combine_SI = TRUE,
  combine_IR = FALSE)

rsi_df(data, translate_ab = "name", language = get_locale(),
  minimum = 30, as_percent = FALSE, combine_SI = TRUE,
  combine_IR = FALSE)
}
\arguments{
\item{...}{one or more vectors (or columns) with antibiotic interpretations. They will be transformed internally with \code{\link{as.rsi}} if needed. Use multiple columns to calculate (the lack of) co-resistance: the probability where one of two drugs have a resistant or susceptible result. See Examples.}

\item{minimum}{the minimum allowed number of available (tested) isolates. Any isolate count lower than \code{minimum} will return \code{NA} with a warning. The default number of \code{30} isolates is advised by the Clinical and Laboratory Standards Institute (CLSI) as best practice, see Source.}

\item{as_percent}{a logical to indicate whether the output must be returned as a hundred fold with \% sign (a character). A value of \code{0.123456} will then be returned as \code{"12.3\%"}.}

\item{only_all_tested}{(for combination therapies, i.e. using more than one variable for \code{...}) a logical to indicate that isolates must be tested for all antibiotics, see section \emph{Combination therapy} below}

\item{data}{a \code{data.frame} containing columns with class \code{rsi} (see \code{\link{as.rsi}})}

\item{translate_ab}{a column name of the \code{\link{antibiotics}} data set to translate the antibiotic abbreviations to, using \code{\link{ab_property}}}

\item{language}{language of the returned text, defaults to system language (see \code{\link{get_locale}}) and can also be set with \code{\link{getOption}("AMR_locale")}. Use \code{language = NULL} or \code{language = ""} to prevent translation.}

\item{combine_SI}{a logical to indicate whether all values of S and I must be merged into one, so the output only consists of S+I vs. R (susceptible vs. resistant). This used to be the parameter \code{combine_IR}, but this now follows the redefinition by EUCAST about the interpretion of I (increased exposure) in 2019, see section 'Interpretation of S, I and R' below. Default is \code{TRUE}.}

\item{combine_IR}{a logical to indicate whether all values of I and R must be merged into one, so the output only consists of S vs. I+R (susceptible vs. non-susceptible). This is outdated, see parameter \code{combine_SI}.}
}
\value{
Double or, when \code{as_percent = TRUE}, a character.
}
\description{
These functions can be used to calculate the (co-)resistance of microbial isolates (i.e. percentage of S, SI, I, IR or R). All functions support quasiquotation with pipes, can be used in \code{dplyr}s \code{\link[dplyr]{summarise}} and support grouped variables, see \emph{Examples}.

\code{portion_R} and \code{portion_IR} can be used to calculate resistance, \code{portion_S} and \code{portion_SI} can be used to calculate susceptibility.\cr
}
\details{
\strong{Remember that you should filter your table to let it contain only first isolates!} This is needed to exclude duplicates and to reduce selection bias. Use \code{\link{first_isolate}} to determine them in your data set.

These functions are not meant to count isolates, but to calculate the portion of resistance/susceptibility. Use the \code{\link[AMR]{count}} functions to count isolates. The function \code{portion_SI()} is essentially equal to \code{count_SI() / count_all()}. \emph{Low counts can infuence the outcome - the \code{portion} functions may camouflage this, since they only return the portion (albeit being dependent on the \code{minimum} parameter).}

The function \code{portion_df} takes any variable from \code{data} that has an \code{"rsi"} class (created with \code{\link{as.rsi}}) and calculates the portions R, I and S. The resulting \emph{tidy data} (see Source) \code{data.frame} will have three rows (S/I/R) and a column for each group and each variable with class \code{"rsi"}.

The function \code{rsi_df} works exactly like \code{portion_df}, but adds the number of isolates.
}
\section{Combination therapy}{

When using more than one variable for \code{...} (= combination therapy)), use \code{only_all_tested} to only count isolates that are tested for all antibiotics/variables that you test them for. See this example for two antibiotics, Antibiotic A and Antibiotic B, about how \code{portion_SI} works to calculate the \%SI:

\preformatted{
-------------------------------------------------------------------------
                        only_all_tested = FALSE   only_all_tested = TRUE
Antibiotic  Antibiotic  -----------------------   -----------------------
    A           B       include as  include as    include as  include as
                        numerator   denominator   numerator   denominator
----------  ----------  ----------  -----------   ----------  -----------
    S           S           X            X             X            X
    I           S           X            X             X            X
    R           S           X            X             X            X
not tested      S           X            X             -            -
    S           I           X            X             X            X
    I           I           X            X             X            X
    R           I           X            X             X            X
not tested      I           X            X             -            -
    S           R           X            X             X            X
    I           R           X            X             X            X
    R           R           -            X             -            X
not tested      R           -            -             -            -
    S       not tested      X            X             -            -
    I       not tested      X            X             -            -
    R       not tested      -            -             -            -
not tested  not tested      -            -             -            -
-------------------------------------------------------------------------
}

Please note that, in combination therapies, for \code{only_all_tested = TRUE} applies that:
\preformatted{
   count_S()  +  count_I()  +  count_R()  == count_all()
  portion_S() + portion_I() + portion_R() == 1
}
and that, in combination therapies, for \code{only_all_tested = FALSE} applies that:
\preformatted{
   count_S()  +  count_I()  +  count_R()  >= count_all()
  portion_S() + portion_I() + portion_R() >= 1
}

Using \code{only_all_tested} has no impact when only using one antibiotic as input.
}

\section{Interpretation of S, I and R}{

In 2019, EUCAST has decided to change the definitions of susceptibility testing categories S, I and R as shown below (\url{http://www.eucast.org/newsiandr/}). Results of several consultations on the new definitions are available on the EUCAST website under "Consultations".

\itemize{
  \item{\strong{S} - }{Susceptible, standard dosing regimen: A microorganism is categorised as "Susceptible, standard dosing regimen", when there is a high likelihood of therapeutic success using a standard dosing regimen of the agent.}
  \item{\strong{I} - }{Susceptible, increased exposure: A microorganism is categorised as "Susceptible, Increased exposure" when there is a high likelihood of therapeutic success because exposure to the agent is increased by adjusting the dosing regimen or by its concentration at the site of infection.}
  \item{\strong{R} - }{Resistant: A microorganism is categorised as "Resistant" when there is a high likelihood of therapeutic failure even when there is increased exposure.}
}

Exposure is a function of how the mode of administration, dose, dosing interval, infusion time, as well as distribution and excretion of the antimicrobial agent will influence the infecting organism at the site of infection.

This AMR package honours this new insight. Use \code{\link{portion_SI}} to determine antimicrobial susceptibility and \code{\link{count_SI}} to count susceptible isolates.
}

\section{Read more on our website!}{

On our website \url{https://msberends.gitlab.io/AMR} you can find \href{https://msberends.gitlab.io/AMR/articles/AMR.html}{a tutorial} about how to conduct AMR analysis, the \href{https://msberends.gitlab.io/AMR/reference}{complete documentation of all functions} (which reads a lot easier than here in R) and \href{https://msberends.gitlab.io/AMR/articles/WHONET.html}{an example analysis using WHONET data}.
}

\examples{
# septic_patients is a data set available in the AMR package. It is true, genuine data.
?septic_patients

# Calculate resistance
portion_R(septic_patients$AMX)
portion_IR(septic_patients$AMX)

# Or susceptibility
portion_S(septic_patients$AMX)
portion_SI(septic_patients$AMX)

# Do the above with pipes:
library(dplyr)
septic_patients \%>\% portion_R(AMX)
septic_patients \%>\% portion_IR(AMX)
septic_patients \%>\% portion_S(AMX)
septic_patients \%>\% portion_SI(AMX)

septic_patients \%>\%
  group_by(hospital_id) \%>\%
  summarise(p = portion_SI(CIP),
            n = n_rsi(CIP)) # n_rsi works like n_distinct in dplyr

septic_patients \%>\%
  group_by(hospital_id) \%>\%
  summarise(R = portion_R(CIP, as_percent = TRUE),
            I = portion_I(CIP, as_percent = TRUE),
            S = portion_S(CIP, as_percent = TRUE),
            n1 = count_all(CIP),  # the actual total; sum of all three
            n2 = n_rsi(CIP),      # same - analogous to n_distinct
            total = n())          # NOT the number of tested isolates!

# Calculate co-resistance between amoxicillin/clav acid and gentamicin,
# so we can see that combination therapy does a lot more than mono therapy:
septic_patients \%>\% portion_SI(AMC)      # \%SI = 76.3\%
septic_patients \%>\% count_all(AMC)       #   n = 1879

septic_patients \%>\% portion_SI(GEN)      # \%SI = 75.4\%
septic_patients \%>\% count_all(GEN)       #   n = 1855

septic_patients \%>\% portion_SI(AMC, GEN) # \%SI = 94.1\%
septic_patients \%>\% count_all(AMC, GEN)  #   n = 1939


# See Details on how `only_all_tested` works. Example:
septic_patients \%>\%
  summarise(numerator = count_SI(AMC, GEN),
            denominator = count_all(AMC, GEN),
            portion = portion_SI(AMC, GEN))
#   numerator denominator portion
#        1764        1936  0.9408
septic_patients \%>\%
  summarise(numerator = count_SI(AMC, GEN, only_all_tested = TRUE),
            denominator = count_all(AMC, GEN, only_all_tested = TRUE),
            portion = portion_SI(AMC, GEN, only_all_tested = TRUE))
#   numerator denominator portion
#       1687        1798   0.9383


septic_patients \%>\%
  group_by(hospital_id) \%>\%
  summarise(cipro_p = portion_SI(CIP, as_percent = TRUE),
            cipro_n = count_all(CIP),
            genta_p = portion_SI(GEN, as_percent = TRUE),
            genta_n = count_all(GEN),
            combination_p = portion_SI(CIP, GEN, as_percent = TRUE),
            combination_n = count_all(CIP, GEN))

# Get portions S/I/R immediately of all rsi columns
septic_patients \%>\%
  select(AMX, CIP) \%>\%
  portion_df(translate = FALSE)

# It also supports grouping variables
septic_patients \%>\%
  select(hospital_id, AMX, CIP) \%>\%
  group_by(hospital_id) \%>\%
  portion_df(translate = FALSE)


\dontrun{

# calculate current empiric combination therapy of Helicobacter gastritis:
my_table \%>\%
  filter(first_isolate == TRUE,
         genus == "Helicobacter") \%>\%
  summarise(p = portion_S(AMX, MTR),  # amoxicillin with metronidazole
            n = count_all(AMX, MTR))
}
}
\seealso{
\code{\link[AMR]{count}_*} to count resistant and susceptible isolates.
}
\keyword{antibiotics}
\keyword{isolate}
\keyword{isolates}
\keyword{resistance}
\keyword{rsi}
\keyword{rsi_df}
\keyword{susceptibility}
